@@ -29,7 +29,7 @@ overlay<- function (..., rlist = NULL, prefix = NULL, sparse=FALSE){
   }
   ##
   if (is.null(dim(dat))) {
-    stop("Please provide a data frame to the overlay function, not a vector.\\n", 
+    stop("Please provide a data frame to the overlay function, not a vector.\\n",
          call. = FALSE)
   }
   if (is.null(rlist)) {
@@ -45,10 +45,10 @@ overlay<- function (..., rlist = NULL, prefix = NULL, sparse=FALSE){
     femlist[[i]] <- ss1[i]
     dat2[, femlist[[i]]] <- as.factor(dat2[, femlist[[i]]])
     if(sparse){
-      S1 <- Matrix::sparse.model.matrix(as.formula(paste("~", femlist[[i]], 
+      S1 <- Matrix::sparse.model.matrix(as.formula(paste("~", femlist[[i]],
                                                          "-1")), dat2)
     }else{
-      S1 <- model.matrix(as.formula(paste("~", femlist[[i]], 
+      S1 <- model.matrix(as.formula(paste("~", femlist[[i]],
                                           "-1")), dat2)
     }
     colnames(S1) <- gsub(femlist[[i]], "", colnames(S1))
@@ -62,17 +62,17 @@ overlay<- function (..., rlist = NULL, prefix = NULL, sparse=FALSE){
   }else{
     S3 <- matrix(0, nrow = dim(dat2)[1], ncol = length(levo))
   }
-  
+
   rownames(S3) <- rownames(dat2)
   colnames(S3) <- levo
   for (i in 1:length(S1list)) {
     if (i == 1) {
-      S3[rownames(S1list[[i]]), colnames(S1list[[i]])] <- S1list[[i]] * 
+      S3[rownames(S1list[[i]]), colnames(S1list[[i]])] <- S1list[[i]] *
         rlist[[i]]
     }
     else {
-      S3[rownames(S1list[[i]]), colnames(S1list[[i]])] <- S3[rownames(S1list[[i]]), 
-                                                             colnames(S1list[[i]])] + (S1list[[i]][rownames(S1list[[i]]), 
+      S3[rownames(S1list[[i]]), colnames(S1list[[i]])] <- S3[rownames(S1list[[i]]),
+                                                             colnames(S1list[[i]])] + (S1list[[i]][rownames(S1list[[i]]),
                                                                                                    colnames(S1list[[i]])] * rlist[[i]])
     }
   }
@@ -84,7 +84,7 @@ overlay<- function (..., rlist = NULL, prefix = NULL, sparse=FALSE){
 }
 
 list2usmat <- function(sigmaL){
-  
+
   f <- function(n, x){
     res <- ((n*(n-1))/2 + n) - x
     if(res < 0){res <- 100}
@@ -93,7 +93,7 @@ list2usmat <- function(sigmaL){
   if(is.list(sigmaL)){
     ss <- unlist(sigmaL)
   }else{ss <- sigmaL}
-  
+
   x <- length(ss)
   n <- round(optimize(f, c(1, 50), tol = 0.0001, x=x)$minimum)
   mss <- matrix(NA,n,n)
@@ -131,7 +131,7 @@ myformula <- function(x){
 }
 
 reshape_mmer <- function(object, namelist){
-  
+
   nt <- nrow(as.matrix(object$sigma[,,1]))
   ntn <- attr(object$sigma, "dimnames")[[2]]
   nre <- length(object$U)
@@ -139,21 +139,21 @@ reshape_mmer <- function(object, namelist){
   U2 <- list()#vector("list",nre)
   VarU2 <- list()
   PevU2 <- list()
-  
+
   if(nre > 0){
     for(ire in 1:nre){
-      
+
       N <- nrow(object$U[[ire]])
       utlist <- list()# vector("list",nt)
       varutlist <- list()# vector("list",nt)
       pevutlist <- list()# vector("list",nt)
-      
+
       # pick <- numeric()
       # for(it in 1:nt){
       #   pick <- c(pick,seq(it,N,nt))
       # }
       provus <- matrix(object$U[[ire]],ncol=nt,byrow = T)
-      
+
       for(it in 1:nt){
         pick <- seq(it,N,nt)
         pit <- ntn[it]
@@ -169,27 +169,27 @@ reshape_mmer <- function(object, namelist){
           rownames(pevutlist[[ pit ]]) <- colnames(pevutlist[[ pit ]]) <- namelist[[ire]]
         }else{pevutlist[[ pit ]] <-pevutlist[[ pit ]]}
       }
-      
+
       U2[[ nren[ire] ]] <- utlist
       VarU2[[ nren[ire] ]] <- varutlist
       PevU2[[ nren[ire] ]] <- pevutlist
     }
-    
+
     object$U <- U2; U2<-NULL
     object$VarU <- VarU2; VarU2<-NULL
     object$PevU <- PevU2; PevU2<-NULL
   }
-  
-  
-  
+
+
+
   N <- nrow(object$Vi)
   pick <- numeric()
   for(it in 1:nt){
     pick <- c(pick,seq(it,N,nt))
   }
-  
+
   object$Vi <- object$Vi[pick,pick]
-  
+
   # object$constraintsF
   # object$Beta <- matrix(object$Beta,ncol=nt,byrow = F)
   # print(namelist[[length(namelist)]])
@@ -197,15 +197,15 @@ reshape_mmer <- function(object, namelist){
   # rownames(object$Beta) <- namelist[[length(namelist)]]
   object$Beta <- cbind(namelist[[length(namelist)]],object$Beta)
   colnames(object$Beta) <- c("Trait","Effect","Estimate")
-  
+
   object$fitted <- matrix(object$fitted,ncol=nt, byrow=TRUE)
   object$residuals <- matrix(object$residuals,ncol=nt, byrow = TRUE)
-  
+
   MyArray <- object$sigma
   object$sigma <- lapply(seq(dim(MyArray)[3]), function(x) MyArray[ , , x])
   object$sigma <- lapply(object$sigma,function(x){mm <- as.matrix(x);colnames(mm)<-rownames(mm) <- ntn;return(mm)})
   names(object$sigma) <- nren
-  
+
   MyArray <- object$sigma_scaled
   object$sigma_scaled <- lapply(seq(dim(MyArray)[3]), function(x) MyArray[ , , x])
   object$sigma_scaled <- lapply(object$sigma_scaled,function(x){mm <- as.matrix(x);colnames(mm)<-rownames(mm) <- ntn;return(mm)})
@@ -218,7 +218,7 @@ reshape_mmer <- function(object, namelist){
 ## na.methods
 
 subdata <- function(data,fixed,na.method.Y=NULL,na.method.X=NULL){
-  
+
   # silently change all columns that are defined as character into factors
   # columnTypes <- unlist(lapply(data, class))
   # columnTypesC <- which(columnTypes == "character")
@@ -283,7 +283,7 @@ subdata <- function(data,fixed,na.method.Y=NULL,na.method.X=NULL){
     data <- data[good,]
   }else{stop("na.method.Y not recognized")}
   data <- data.frame(data)
-  
+
   ##########
   ## na.method x
   yuyu <- strsplit(as.character(fixed[3]), split = "[+]")[[1]]
@@ -292,13 +292,13 @@ subdata <- function(data,fixed,na.method.Y=NULL,na.method.X=NULL){
   })
   xtermss2 <- apply(data.frame(xtermss),1,function(x){gsub(",.*","",expi2(x))})
   xtermss2[which(xtermss2 == "")] <- xtermss[which(xtermss2 == "")]
-  
+
   xtermss2 <- intersect(colnames(data),xtermss2) # only focus on the terms that are in teh dataset so we can skip overlay and weird vs structures
-  
+
   # print(xtermss2)
   if(length(xtermss2) > 0){
     mycl <- as.vector(unlist(lapply(data.frame(data[,xtermss2]),class)))
-    
+
     if(na.method.X=="include"){
       touse <- xtermss2
       for(i in 1:length(touse)){
@@ -325,13 +325,13 @@ subdata <- function(data,fixed,na.method.Y=NULL,na.method.X=NULL){
     }else{stop("na.method.Y not recognized")}
     data <- data.frame(data)
   }
-  
+
   return(list(datar=data,good=good))
-  
+
 }
 
-############## 
-## VS structure
+##############
+## VS structures
 atr <- function(x, levs){
   if(is.matrix(x)){
     dummy <- x
@@ -379,7 +379,7 @@ csr <- function(x,mm){
         dummy  <- model.matrix(~dummy-1,na.action = na.pass)
         colnames(dummy) <- gsub("dummy","",colnames(dummy))
       }else{
-        vv <- which(!is.na(dummy)); 
+        vv <- which(!is.na(dummy));
         dummy <- matrix(0,nrow=length(dummy))
         dummy[vv,] <- 1; colnames(dummy) <- levs
       }
@@ -398,7 +398,7 @@ dsr <- function(x){
     if(!is.character(x) & !is.factor(x)){
       namess <- as.character(substitute(list(x)))[-1L]
       dummy <- matrix(x,ncol=1); colnames(dummy) <- namess
-      mm <- diag(ncol(dummy)); 
+      mm <- diag(ncol(dummy));
     }else{
       dummy <- x
       levs <- na.omit(unique(dummy))
@@ -406,7 +406,7 @@ dsr <- function(x){
         dummy  <- model.matrix(~dummy-1,na.action = na.pass)
         colnames(dummy) <- gsub("dummy","",colnames(dummy))
       }else{
-        vv <- which(!is.na(dummy)); 
+        vv <- which(!is.na(dummy));
         dummy <- matrix(0,nrow=length(dummy))
         dummy[vv,] <- 1; colnames(dummy) <- levs
       }
@@ -432,19 +432,67 @@ usr <- function(x){
         dummy  <- model.matrix(~dummy-1,na.action = na.pass)
         colnames(dummy) <- gsub("dummy","",colnames(dummy))
       }else{
-        vv <- which(!is.na(dummy)); 
+        vv <- which(!is.na(dummy));
         dummy <- matrix(0,nrow=length(dummy))
         dummy[vv,] <- 1; colnames(dummy) <- levs
       }
     }
     mm <- unsm(ncol(dummy))
   }
-  colnames(mm) <- rownames(mm) <- colnames(dummy) 
+  colnames(mm) <- rownames(mm) <- colnames(dummy)
   return(list(Z=dummy,thetaC=mm))
 }
 
 ###############
-## VS structure for mmec
+## VS structures for mmec
+
+rrc <- function(timevar=NULL, idvar=NULL, response=NULL, Gu=NULL, nPC=2, returnLam=FALSE, cholD=TRUE){
+  if(is.null(timevar)){stop("Please provide the timevar argument.", call. = FALSE)}
+  if(is.null(idvar)){stop("Please provide the idvar argument.", call. = FALSE)}
+  if(is.null(response)){stop("Please provide the response argument.", call. = FALSE)}
+  # these are called PC models by Meyer 2009, GSE. This is a reduced rank implementation
+  # we produce loadings, the Z*L so we can use it to estimate factor scores in mmec()
+  dtx <- data.frame(timevar=timevar, idvar=idvar, v.names=response)
+  dtx2 <- aggregate(v.names~timevar+idvar, data=dtx, FUN=mean, na.rm=TRUE)
+  wide <- reshape(dtx2, direction = "wide", idvar = "idvar",
+                  timevar = "timevar", v.names = "v.names", sep= "_")
+  Y <- apply(wide[,-1],2, sommer::imputev)
+  rownames(Y) <- wide[,1]
+  if(!is.null(Gu)){Y=Gu%*%Y} # adjust if covariance matrix exist
+  GE <- cov(scale(Y, scale = TRUE, center = TRUE)) # surrogate of unstructured matrix to start with
+  GE <- as.matrix(nearPD(GE)$mat)
+  # GE <- as.data.frame(t(scale( t(scale(Y, center=T,scale=F)), center=T, scale=F)))  # sum(GE^2)
+  if(cholD){
+    ## OPTION 2. USING CHOLESKY
+    Lam <- t(chol(GE)); # LOADINGS  # same GE=LL' from cholesky  plot(unlist(Lam%*%t(Lam)), unlist(GE))
+  }else{
+    ## OPTION 1. USING SVD
+    U <- svd(GE)$u;  # V <- svd(GE)$v
+    D <- diag(svd(GE)$d)
+    Lam <- U %*% sqrt(D); # LOADINGS 
+  }
+  # pick required PCs
+  colnamesLam <- colnames(Lam)
+  Lam <- as.matrix(Lam[,1:nPC]); colnames(Lam) <- colnamesLam[1:nPC]  # Se <- Se[,1:nPC]
+  ##
+  rownames(Lam) <- rownames(GE)#levels(dataset$Genotype);  # rownames(Se) <- colnames(GE)#levels(dataset$Environment)
+  colnames(Lam) <- paste("PC", 1:ncol(Lam), sep =""); # colnames(Se) <- paste("PC", 1:ncol(Se), sep ="")
+  rownames(Lam) <- gsub("v.names_","", rownames(Lam))
+  ######### GEreduced = Sg %*% t(Se) 
+  # if we want to merge with PCs for environments
+  dtx$index <- 1:nrow(dtx)
+  dtx2 <- dtx[which(!is.na(dtx$v.names)),]
+  Z <- Matrix::sparse.model.matrix(~timevar -1, na.action = na.pass, data=dtx2)
+  colnames(Z) <- gsub("timevar","",colnames(Z))
+  Z <- Z%*%Lam[colnames(Z),] # we multiple original Z by the LOADINGS
+  Z <- as.matrix(Z)
+  rownames(Z) <- NULL
+  if(returnLam){
+    return(list(Lam=Lam))
+  }else{
+    return(Z)
+  }
+}
 atc <- function(x, levs, thetaC=NULL, theta=NULL){
   if(is.matrix(x)){
     dummy <- x
@@ -506,7 +554,7 @@ csc <- function(x,mm, thetaC=NULL, theta=NULL){
         dummy  <- Matrix::sparse.model.matrix(~dummy-1,na.action = na.pass)
         colnames(dummy) <- gsub("dummy","",colnames(dummy))
       }else{
-        vv <- which(!is.na(dummy)); 
+        vv <- which(!is.na(dummy));
         dummy <- matrix(0,nrow=length(dummy))
         dummy[vv,] <- 1; colnames(dummy) <- levs
       }
@@ -536,7 +584,7 @@ dsc <- function(x, thetaC=NULL, theta=NULL){
     if(!is.character(x) & !is.factor(x)){
       namess <- as.character(substitute(list(x)))[-1L]
       dummy <- as(Matrix(x,ncol=1), Class = "dgCMatrix"); colnames(dummy) <- namess
-      mm <- diag(ncol(dummy)); 
+      mm <- diag(ncol(dummy));
     }else{
       dummy <- x
       levs <- na.omit(unique(dummy))
@@ -544,7 +592,7 @@ dsc <- function(x, thetaC=NULL, theta=NULL){
         dummy  <- Matrix::sparse.model.matrix(~dummy-1,na.action = na.pass)
         colnames(dummy) <- gsub("dummy","",colnames(dummy))
       }else{
-        vv <- which(!is.na(dummy)); 
+        vv <- which(!is.na(dummy));
         dummy <- matrix(0,nrow=length(dummy))
         dummy[vv,] <- 1; colnames(dummy) <- levs
       }
@@ -581,19 +629,15 @@ usc <- function(x, thetaC=NULL, theta=NULL){
         dummy  <- Matrix::sparse.model.matrix(~dummy-1,na.action = na.pass)
         colnames(dummy) <- gsub("dummy","",colnames(dummy))
       }else{
-        vv <- which(!is.na(dummy)); 
+        vv <- which(!is.na(dummy));
         dummy <- Matrix(0,nrow=length(dummy))
         dummy[vv,] <- 1; colnames(dummy) <- levs
       }
     }
     mm <- unsm(ncol(dummy))
   }
-  colnames(mm) <- rownames(mm) <- colnames(dummy) 
+  colnames(mm) <- rownames(mm) <- colnames(dummy)
   bnmm <- diag(ncol(mm))*.05 + matrix(.1,ncol(mm),ncol(mm))
-  # if(nrow(bnmm) > 1){
-  #   bnmm[upper.tri(bnmm)]=bnmm[upper.tri(bnmm)]/3
-  #   bnmm[lower.tri(bnmm)]=bnmm[lower.tri(bnmm)]/3
-  # }
   if(!is.null(thetaC)){
     mm <- thetaC
   }
@@ -611,18 +655,18 @@ isc <- function(x, thetaC=NULL, theta=NULL){
     if(!is.character(x) & !is.factor(x)){
       namess <- as.character(substitute(list(x)))[-1L]
       dummy <- Matrix(x,ncol=1); colnames(dummy) <- namess
-      mm <- diag(1); 
+      mm <- diag(1);
     }else{
       dummy <- x
       levs <- na.omit(unique(dummy))
       if(length(levs) > 1){
-        dummy  <- Matrix::sparse.model.matrix(~dummy-1,na.action = na.pass) 
+        dummy  <- Matrix::sparse.model.matrix(~dummy-1,na.action = na.pass)
         if(!is(class(dummy), "dgCMatrix")){
-          dummy <- as(dummy, Class="dgCMatrix") 
+          dummy <- as(dummy, Class="dgCMatrix")
         }
         colnames(dummy) <- gsub("dummy","",colnames(dummy))
       }else{
-        vv <- which(!is.na(dummy)); 
+        vv <- which(!is.na(dummy));
         dummy <- Matrix(0,nrow=length(dummy))
         dummy[vv,] <- 1; colnames(dummy) <- levs
       }
@@ -670,28 +714,28 @@ fcm <- function(x, reps=NULL){
 }
 
 bivariateRun <- function(model, n.core=1){
-  
+
   args <- model[[length(model)]]
-  
+
   if(!args$return.param){
     stop("The model provided needs to have the return.param argument set to TRUE. \nPlease read the documentation of the bivariateRun function carefully.\n", call. = FALSE)
   }
-  
+
   response <- strsplit(as.character(args$fixed[2]), split = "[+]")[[1]]
   expi <- function(j){gsub("[\\(\\)]", "", regmatches(j, gregexpr("\\(.*?\\)", j))[[1]])}
   expi2 <- function(x){gsub("(?<=\\()[^()]*(?=\\))(*SKIP)(*F)|.", "", x, perl=T)}
   traits <- trimws(strsplit(expi(response),",")[[1]])
-  
+
   combos <- expand.grid(traits,traits)
-  combos <- combos[which(combos[,1] != combos[,2]),]; 
+  combos <- combos[which(combos[,1] != combos[,2]),];
   combos <- combos[!duplicated(t(apply(combos, 1, sort))),];rownames(combos) <- NULL
-  
+
   RHS <- as.character(args$fixed[3])
   it <- as.list(1:nrow(combos))
-  
+
   cat(paste(nrow(combos), "bivariate models to be run\n"))
-  
-  model.results <- parallel::mclapply(it, 
+
+  model.results <- parallel::mclapply(it,
                                       function(x) {
                                         # score.calc(M[ix.pheno, markers])
                                         ff <- as.formula(paste("cbind(",paste(as.vector(unlist(combos[x,])),collapse = ","),") ~", RHS))
@@ -707,13 +751,13 @@ bivariateRun <- function(model, n.core=1){
                                         p1 <- gsub("diag\\([[:digit:]])","diag(2)",p1)
                                         p1 <- gsub("uncm\\([[:digit:]])","uncm(2)",p1)
                                         args2$rcov <- as.formula(paste(p1[1],paste(p1[-1],collapse = "+")))
-                                        
+
                                         gsub("[1-9]","k",as.character(args2$random))
                                         res0 <- do.call(mmer, args=args2)
                                         return(res0)
-                                      }, 
+                                      },
                                       mc.cores = n.core)
-  
+
   sigmas <- lapply(model.results, function(x){x$sigma})
   sigmas_scaled <- lapply(model.results, function(x){x$sigma_scaled})
   nre <- length(sigmas[[1]])
@@ -733,7 +777,7 @@ bivariateRun <- function(model, n.core=1){
     sigmaslist[[namesre[i]]] <- mt
     sigmas_scaledlist[[namesre[i]]] <- mts
   }
-  
+
   names(model.results) <- apply(combos,1,function(x){paste(x,collapse = "-")})
   # corlist <- lapply(sigmaslist,cov2cor)
   final <- list(sigmas=sigmaslist, sigmas_scaled=sigmas_scaledlist, models=model.results)
@@ -767,14 +811,14 @@ transformConstraints <- function(list0,value=1){
 #   base[lower.tri(base, diag=TRUE)] <- 1:length(l)
 #   index <- which(!is.na(base), arr.ind = TRUE)
 #   index <- index[order(index[,1]), ]
-#   
-#   
+#
+#
 #   for(i in 1:ntrue){ # for each main blup
 #     main <- which(index[,1] == i & index[,2] == i, arr.ind = TRUE)
 #     cov1 <- which(index[,1] == i & index[,2] != i, arr.ind = TRUE)
 #     cov2 <- which(index[,1] != i & index[,2] == i, arr.ind = TRUE)
 #     for(itrait in 1:ntraits){
-#       start <- blups[[main]][[itrait]]  
+#       start <- blups[[main]][[itrait]]
 #       for(icov1 in cov1){
 #         start <- start + blups[[icov1]][[itrait]][indexCov1]
 #       }
