@@ -1,15 +1,15 @@
 vsc <- function(..., Gu=NULL, buildGu=TRUE, meN=1, meTheta=NULL, meThetaC=NULL, sp=FALSE){
-  
+
   ## ... list of structures to define the random effect , e.g. init <- list(ds(M$data$FIELD),TP)
   ## Gu the known covariance matrix of the vs
-  
+
   init <- list(...) #  e.g. init <- list(usc(data$Env),isc(data$Name)) | init <- list(dsc(data$YEAR),isc(data$units))
-  
+
   namess <- as.character(substitute(list(...)))[-1L] # namess <- c("Env","Name") | namess <- c("YEAR","units")
-  namess2 <- apply(data.frame(namess),1,function(x){ 
+  namess2 <- apply(data.frame(namess),1,function(x){
     return(all.vars(as.formula(paste0("~",x))))
   })
-  
+
   ## let's test that user provided all terms encapsulated in a structure
   listLength <- length(init)
   if(listLength == 1){ # very simple structure
@@ -22,22 +22,23 @@ vsc <- function(..., Gu=NULL, buildGu=TRUE, meN=1, meTheta=NULL, meThetaC=NULL,
     whichElemBad <- which(lengthElem < 3) ## which are not encapsulated
   }
   if(length(whichElemBad) > 0){
-    stop(paste("Term(s): '",paste(namess2[whichElemBad],collapse = ","),"' in the vsc() function are not encapsulated in a structure function. Please correct (e.g., using isc(), dsc(), usc(), atc(), csc(), etc.)."),call. = FALSE)
+    badd <- paste(namess2[whichElemBad],collapse = ",")
+    stop(paste0("Term(s): '",badd,"' in the vsc() function are not encapsulated in a structure function. Please correct [for example, using vsc(isc(",badd,")), vsc(dsc(",badd,")), vsc(usc(",badd,")), vsc(atc(",badd,")), vsc(csc(",badd,")), etc.]."),call. = FALSE)
   }
-  
+
   ## extract names of variables and collpase as interaction
-  
+
   namess2 <- as.vector(unique(unlist(namess2))) # remove repeats if exist
   namess2[which(namess2 == "")] <- namess[which(namess2 == "")] # remove empties if exist
   # extract the name of the main variable
   ref_name <- namess2[length(namess2)]
   # certain random effects coming from spl2D(), leg(), and others may need some help to find the terms
   specialVariables <- unlist(lapply(init,function(x){(attributes(x)$variables)}))
-  # 
+  #
   if("units" %in% namess2){ # if residual
     is.residual =TRUE
   }else{is.residual=FALSE}
-  
+
   ### get the data
   if(length(init)>meN){ # if there is a covariance structure not only isc
     for(i in 1:(length(init)-meN)){ # for all terms prior to the main effect(s) keep nme in mind
@@ -67,6 +68,24 @@ vsc <- function(..., Gu=NULL, buildGu=TRUE, meN=1, meTheta=NULL, meThetaC=NULL,
     Z0 <- Matrix(1,nrow(init[[1]]$Z),1)
     pasteNames=FALSE
   }
+  #######################################
+  ## check covariance matrices
+  `%!in%` = Negate(`%in%`)
+  if(is.null(Gu)){
+    x <- data.frame(d=as.factor(1:ncol(init[[length(init)]]$Z)))
+    Gu <- sparse.model.matrix(~d-1, x)
+    colnames(Gu) <- rownames(Gu) <- colnames(init[[length(init)]]$Z)
+  }else{
+    if (!inherits(Gu, "dgCMatrix")){
+      stop("Gu matrix is not of class dgCMatrix. Please correct \n", call. = TRUE )
+    }
+    # cn <- colnames(init[[k]]$Z)
+    # if(is.null(cn)){
+    #   stop("Gu matrix needs to have row and column names matching the levels of the random effect. Please correct \n", call. = TRUE )
+    # }
+    # Gu <- Gu[cn,cn]
+  }
+  #############################
   ######################################
   ## now build the Z matrices
   Z <- list()
@@ -78,6 +97,7 @@ vsc <- function(..., Gu=NULL, buildGu=TRUE, meN=1, meTheta=NULL, meThetaC=NULL,
     # baseZnames <- colnames(init[[1]]$Z)
     for(k in (length(init)-meN+1):(length(init))){
       Z1prov <- init[[k]]$Z # main effect matrix
+      cn <- colnames(Z1prov)
       if(is.residual){
         vv <- which(Z0[,j] != 0) # which rows belong to the ith environment
         partitionsR[[j]] <- matrix(c(vv[1],vv[length(vv)]),nrow=1)
@@ -91,9 +111,25 @@ vsc <- function(..., Gu=NULL, buildGu=TRUE, meN=1, meTheta=NULL, meThetaC=NULL,
         }else{
           Z[[counter]] <- Z1provZ0iCol
         }
+        if(is.null(cn)){
+          stop("Gu matrix needs to have row and column names matching the levels of the random effect. Please correct \n", call. = TRUE )
+        }
+        # print(cn)
+        # print(colnames(Gu))
+        checkg <- setdiff(cn,colnames(Gu)) # make sure missing levels is not a thing
+        # print(checkg)
+        if(length(checkg)>0){
+          stop(paste("levels of",ref_name,"missing in Gu"),call. = FALSE)
+        }
+        checkg2 <- setdiff(colnames(Gu),cn) # check if additional G levels exist
+        if(length(checkg2)>0){
+          cat(paste0("Adding additional levels of Gu in the model matrix of '",ref_name,"' \n"))
+          added <- Matrix(0, nrow = nrow(Z1provZ0iCol), ncol = length(checkg2)); colnames(added) <- checkg2
+          Z[[counter]] <- cbind(Z1provZ0iCol,added)
+        }
       }
       counter <- counter+1
-      if(meN <= 1){ # if there's only one main effect 
+      if(meN <= 1){ # if there's only one main effect
         if(pasteNames & !is.residual){ # if there's a covariance structure paste names, if only isc() don't
           # colnames(Z[[j]]) <- paste(colnames(Z1prov),colnames(Z0)[j],sep=":")
         }
@@ -101,6 +137,7 @@ vsc <- function(..., Gu=NULL, buildGu=TRUE, meN=1, meTheta=NULL, meThetaC=NULL,
     }
   }
   Zind <- rep(1,length(Z))
+
   ######################################
   ## meN adjustment
   ## modify theta and thetaC according to the number of mainEffect matrices
@@ -114,24 +151,7 @@ vsc <- function(..., Gu=NULL, buildGu=TRUE, meN=1, meTheta=NULL, meThetaC=NULL,
     theta <- kronecker(theta,meTheta, make.dimnames = TRUE)
     thetaC <- kronecker(thetaC,meThetaC, make.dimnames = TRUE)
   }
-  #######################################
-  ## check covariance matrices
-  `%!in%` = Negate(`%in%`)
-  if(is.null(Gu)){
-    x <- data.frame(d=as.factor(1:ncol(Z[[1]])))
-    Gu <- sparse.model.matrix(~d-1, x)
-    colnames(Gu) <- rownames(Gu) <- colnames(Z[[1]])
-  }else{
-    if (!inherits(Gu, "dgCMatrix")){
-      stop("Gu matrix is not of class dgCMatrix. Please correct \n", call. = TRUE )
-    }
-    cn <- colnames(Z[[1]])
-    if(is.null(cn)){
-      stop("Gu matrix needs to have row and column names matching the levels of the random effect. Please correct \n", call. = TRUE )
-    }
-    # print(cn)
-    Gu <- Gu[cn,cn]
-  }
+
   #########################################
   ## thetaF
   nn <- length(which(thetaC > 0))#unlist(lapply(thetaC, function(x){length(which(x > 0))}))
@@ -141,7 +161,7 @@ vsc <- function(..., Gu=NULL, buildGu=TRUE, meN=1, meTheta=NULL, meThetaC=NULL,
   sp0 <- ifelse(sp,1,0)
   sp0 <- rep(sp0,nrow(thetaF))
   if(sp){thetaF <- thetaF*0}
-  
+
   output <- list(Z=Z, Gu=Gu, theta=theta, thetaC=thetaC, thetaF=thetaF,partitionsR=partitionsR, sp=sp0)
   return(output)
 }
